#!/bin/bash

# Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
# 
# This software is published by the license of CPU-OS License, you can use and 
# distibuted this software under this License. See CPU-OS Liense for more detail.
#
# You should have receive a copy of CPU-OS License. If not, please contact us 
# by email <support_os@cpu-os.ac.cn>

set -e
get_kernel_name(){
  kernel_release=`uname -r`
# echo $kernel_version
  kernel_name="/boot/vmlinuz-"$kernel_release
  # kernel_name="test-vmlinuz"
# echo $kernel_name
 if [ -f $kernel_name ];then
     echo $kernel_name
 else
    # kernel_name is not exist,find it from /boot
    kernel_name=`ls /boot | grep "vmlinuz" | grep "$kernel_release" | head -n1`
    # kernel_name=""
    if [ ! -z "$kernel_name" ];then
    	 kernel_name="/boot/"$kernel_name
    	 echo $kernel_name
    else
          if [ -f /boot/grub/grub.cfg ]; then
              kernel_name=`cat /boot/grub/grub.cfg | grep linux | grep root= | grep vmlinuz | awk '{print $2}' | head -n1`
    	      echo $kernel_name
          else
    	      kernel_name=`ls /boot | grep "vmlinuz" | head -n1`
    	      # kernel_name=""
    	      if [ ! -z "$kernel_name" ];then
    	         kernel_name="/boot/"$kernel_name
    	         echo $kernel_name
    	      else
    	         echo ""
    	      fi
          fi
    fi
 fi
}

get_initrd_name(){
  kernel_release=`uname -r`
  initrd_name="/boot/initrd.img-"$kernel_release
  # initrd_name="test-vmlinuz"
 if [ -f $initrd_name ];then
     echo $initrd_name
 else
    # initrd_name is not exist,find it from /boot
    initrd_name=`ls /boot | grep "initrd.img" | grep "$kernel_release"`
    # initrd_name=""
    if [ ! -z "$initrd_name" ];then
    	 initrd_name="/boot/"$initrd_name
    	 echo $initrd_name
    	else
    	  initrd_name=`ls /boot | grep "initrd.img"`
    	  # initrd_name=""
    	  if [ ! -z "$initrd_name" ];then
    	    initrd_name="/boot/"$initrd_name
    	    echo $initrd_name
    	  else
    	    echo ""
    	  fi
    fi
 fi
}

set_boot_dir(){
  # efi_dir=`cat /etc/os-release | grep "CODENAME=" | awk -F '=' '{print $2}'`
  boot_dir="/boot/efi/EFI/cdos"
  # echo $boot_dir $efi_dir
  if [ ! -d $boot_dir ];then
    mkdir -p $boot_dir
  fi
  return 0
}

copy_kernel_to_efi(){
  cp $1 /boot/efi/EFI/cdos/vmlinuz.efi
}

copy_initrd_to_efi(){
  cp $1 /boot/efi/EFI/cdos/initrd.img
}

get_grub_cmd_linux(){
  sysconfdir="/etc"
  if test -f ${sysconfdir}/default/grub ; then
    . ${sysconfdir}/default/grub
  fi
  for x in ${sysconfdir}/default/grub.d/*.cfg ; do
    if [ -e "${x}" ]; then
      . "${x}"
    fi
  done
  echo "$GRUB_CMDLINE_LINUX $GRUB_CMDLINE_LINUX_DEFAULT"
}

set_efi_part(){
#get efi part msg
  efi_part=`mount | grep "on /boot/efi" | awk -F ' ' '{print $1}'`
  efi_part_name=`echo $efi_part | awk -F '[0-9]' '{print $1}'`
  efi_part_id=`echo ${efi_part#*$efi_part_name}`
  #If partation is nvme, the first part is /dev/nvme0n1p1,...
  #This is different with /dev/sda1, so need the follow lines
  if [ "X$efi_part_name" = "X/dev/nvme" ]; then
     efi_part_name=`echo $efi_part | awk -F 'p[0-9]$' '{print $1}'`
     efi_part_name2=`echo $efi_part | awk -F '[0-9]$' '{print $1}'`
     efi_part_id=`echo ${efi_part#*$efi_part_name2}`
  fi
  #echo "Part name: $efi_part_name , ID: $efi_part_id "

#get root part msg
  root_part=`mount | grep "on / " | awk -F ' ' '{print $1}'`
#get swap part msg
  #swap_part=`cat /etc/fstab | grep ^"# swap" | sed -n "1,1p" | awk -F ' ' '{print $5}'`
  #For the installer changed the content of /etc/fstab, the swap_part need changed
  swap_part=`blkid  | grep TYPE=\"swap\" | awk -F: '{print $1}'`
# get root fs type
  rootfstype=`mount | grep "on / " | awk -F ' ' '{print $5}'`
  grub_cmd_linux=`get_grub_cmd_linux | sed -e 's/hibernate=nocompress//g' | sed -e 's/hibernation=nocompress//g' | sed -e 's/init=\/sbin\/e4rat-preload//g'`
  case $1 in
    withinitrd)
      case $2 in
        without-resume)
          echo " root=$root_part ro rootfstype=$rootfstype $grub_cmd_linux initrd=/EFI/cdos/initrd.img raid=noautodetect loglevel=0 systemd.show_status=0" | iconv -t ucs2 | efibootmgr -c -g -d $efi_part_name -p $efi_part_id -L "cdos-quickstart" -l '/EFI/cdos/vmlinuz.efi' -@ -
          ;;
        *)
          echo " root=$root_part ro rootfstype=$rootfstype $grub_cmd_linux resume=$swap_part initrd=/EFI/cdos/initrd.img raid=noautodetect loglevel=0 systemd.show_status=0" | iconv -t ucs2 | efibootmgr -c -g -d $efi_part_name -p $efi_part_id -L "cdos-quickstart" -l '/EFI/cdos/vmlinuz.efi' -@ -
          ;;
      esac
      ;;
    withoutinitrd)
      case $2 in
        without-resume)
          echo " root=$root_part ro rootfstype=$rootfstype $grub_cmd_linux raid=noautodetect loglevel=0 systemd.show_status=0" | iconv -t ucs2 | efibootmgr -c -g -d $efi_part_name -p $efi_part_id -L "cdos-quickstart" -l '/EFI/cdos/vmlinuz.efi' -@ -
	  ;;
        *)
          echo " root=$root_part ro rootfstype=$rootfstype $grub_cmd_linux resume=$swap_part raid=noautodetect loglevel=0 systemd.show_status=0" | iconv -t ucs2 | efibootmgr -c -g -d $efi_part_name -p $efi_part_id -L "cdos-quickstart" -l '/EFI/cdos/vmlinuz.efi' -@ -
          ;;
      esac
      ;;
  esac
  # sudo efibootmgr -d $efi_part_name -c -p $efi_part_id -L "cdos-linux" -l /EFI/cdos/vmlinuz.efi -u "root=$root_part ro rootfstype=ext4 init=/sbin/e4rat-preload splash quiet resume=$swap_part"
  return 0
}

set_quick_start_with_initrd(){
  set_boot_dir
  kernel=`get_kernel_name`
  if [ ! -z "$kernel" ];then
    echo "get kernel:  "$kernel
    copy_kernel_to_efi $kernel 
    initrd=`get_initrd_name`
    if [ ! -z "$initrd" ];then
      echo "get initrd:  "$initrd
      copy_initrd_to_efi $initrd
      set_efi_part withinitrd
    else
    	echo "initrd is not exist,set quick start without initrd"
    	  set_efi_part withoutinitrd
    fi
    return 0
  else
    echo "kernel is not exist"
    return 1
  fi

# echo $?
}

set_quick_start_without_initrd(){
  set_boot_dir
  kernel=`get_kernel_name`
  if [ ! -z "$kernel" ];then
    echo "get kernel:  "$kernel
    copy_kernel_to_efi $kernel 
    set_efi_part withoutinitrd
    return 0
  else
    echo "kernel is not exist"
    return 1
  fi
}

get_grub_cmd_linux_line(){
  sysconfdir="/etc"
  if test -f ${sysconfdir}/default/grub ; then
    . ${sysconfdir}/default/grub
  fi
  for x in ${sysconfdir}/default/grub.d/*.cfg ; do
    if [ -e "${x}" ]; then
      . "${x}"
    fi
  done
  cmd_lines=`echo "$GRUB_CMDLINE_LINUX_DEFAULT"`
  for cmd_line in $cmd_lines ; do
    if [[ ${cmd_line:0:6} == resume ]] || [[ ${cmd_line:0:10} == rootfstype ]];then
      :
    else
      echo $cmd_line
    fi
  done
}

get_last_grub_cfg_file(){
  sysconfdir="/etc"
  if test -f ${sysconfdir}/default/grub ; then
    grub_cfg=${sysconfdir}/default/grub
  fi
  for x in ${sysconfdir}/default/grub.d/*.cfg ; do
    if [ -e "${x}" ]; then
      grub_cfg="${x}"
    fi
  done
  echo $grub_cfg
}

set_fastboot_on(){
  if ! check_can_use_for_fastboot;then
    return 1
  fi

  if [ -d /sys/firmware/efi ];then
    efiloader_set add
  fi

  #get root part msg
  root_part=`mount | grep "on / " | awk -F ' ' '{print $1}'`
  grub_cmd_linux=`get_grub_cmd_linux_line`
  grub_cfg_file=`get_last_grub_cfg_file`

  # update grub_cfg in /etc/default
  if [ ! -z "$grub_cfg_file" ];then
    if [ -f "$grub_cfg_file"_bk ];then
      rm "$grub_cfg_file"_bk
    fi
    while read line
    do
      if [[ ! ${line:0:27} == "GRUB_CMDLINE_LINUX_DEFAULT=" ]] && [[ ! ${line:0:24} == "GRUB_DISABLE_LINUX_UUID=" ]];then
        echo $line >> "$grub_cfg_file"_bk
      fi
    done < $grub_cfg_file
    echo "GRUB_CMDLINE_LINUX_DEFAULT=\""$grub_cmd_linux"\"" >> "$grub_cfg_file"_bk
    echo "GRUB_DISABLE_LINUX_UUID=true" >> "$grub_cfg_file"_bk
    cp "$grub_cfg_file"_bk "$grub_cfg_file"
  else
    sysconfdir="/etc"
    echo "GRUB_CMDLINE_LINUX_DEFAULT=\""$grub_cmd_linux"\"" > ${sysconfdir}/default/grub
    echo "GRUB_DISABLE_LINUX_UUID=true" >> ${sysconfdir}/default/grub
  fi

  # when we append GRUB_GFXPAYLOAD_LINUX to any file under dir /etc/default/grub.d/, it takes no effect. So here we ...
  # cp -f /usr/share/cdos-fastboot/cdos-loader /boot/

  if check_env_for_zxd
  then
    /bin/bash /usr/share/cdos-fastboot/fstab_modi.sh
  fi

  python /usr/share/cdos-fastboot/fastboot.py -s on -r $root_part
  sed -i 's/STATE=off/STATE=on/g' /etc/cdos-fastboot/cdos-fastboot.conf
}

set_fastboot_off(){
  # bios_version = `dmidecode -s bios-version`
  # if [ ! -d /boot/efi/EFI/Boot ] && [ -d /boot/efi/EFI/Boot.bak ]
  # then
  #   mv /boot/efi/EFI/Boot.bak /boot/efi/EFI/Boot
  # fi
  if [ -d /sys/firmware/efi ];then
    efiloader_set remove
  fi

  /bin/bash /usr/share/cdos-fastboot/fstab_modi.sh reset

  python /usr/share/cdos-fastboot/fastboot.py -s off
  sed -i 's/STATE=on/STATE=off/g' /etc/cdos-fastboot/cdos-fastboot.conf
}

set_efi_quick_start_on(){
  if ! check_can_use_for_fastboot;then
    return 1
  fi
  efibootmgr | grep "cdos-quickstart" | awk -F ' ' '{print $1}' | while read boot_order_line
  do
    boot_order=`echo ${boot_order_line%*\*}`
    boot_order_num=`echo ${boot_order#*Boot}`
    efibootmgr -B -b $boot_order_num
  done
  case $1 in
    initrd)
      set_quick_start_with_initrd
      ;;
    noinitrd)
      set_quick_start_without_initrd
      ;;
    # *)
    #   echo $"Usage: $0 {initrd|noinitrd}"
    #   ;;
  esac
  # inactive no quick bootnum
  case $2 in
    updatebootorder)
      efibootmgr | grep -iE "ip|sata" | awk -F ' ' '{print $1}' | while read boot_order_line
      do
        if [[ ${boot_order_line:8:1}x == "*x" ]];then
          boot_order=`echo ${boot_order_line%*\*}`
          boot_order_num=`echo ${boot_order#*Boot}`
          efibootmgr -A -b $boot_order_num
        fi
      done
      sed -i 's/STATE=off/STATE=on/g' /etc/cdos-fastboot/cdos-fastboot.conf
      ;;
    *)
      ;;
  esac
}

reset_fastboot_state(){
  num=`efibootmgr | grep "cdos-quickstart" | awk -F ' ' '{print $1}' | wc -l`
  if [ $num -eq 0 ];then
    sed -i 's/STATE=on/STATE=off/g' /etc/cdos-fastboot/cdos-fastboot.conf
    return
  fi
  efibootmgr | grep "cdos-quickstart" | awk -F ' ' '{print $1}' | while read boot_order_line
  do
    if [[ ${boot_order_line:8:1}x == "*x" ]];then
      sed -i 's/STATE=off/STATE=on/g' /etc/cdos-fastboot/cdos-fastboot.conf
    elif [[ ${boot_order_line:8:1}x == "x" ]];then
      sed -i 's/STATE=on/STATE=off/g' /etc/cdos-fastboot/cdos-fastboot.conf
    fi
  done
}

reset_efi_bootorder(){
  # inactive no quick bootnum
  efibootmgr | grep -iE "ip|sata" | awk -F ' ' '{print $1}' | while read boot_order_line
  do
    if [[ ${boot_order_line:8:1}x == "*x" ]];then
      boot_order=`echo ${boot_order_line%*\*}`
      boot_order_num=`echo ${boot_order#*Boot}`
      efibootmgr -A -b $boot_order_num
    fi
  done
}

# set_efi_quick_start_on initrd
set_efi_quick_start_off(){
  efibootmgr | grep "cdos-quickstart" | awk -F ' ' '{print $1}' | while read boot_order_line
  do
    boot_order=`echo ${boot_order_line%*\*}`
    boot_order_num=`echo ${boot_order#*Boot}`
    efibootmgr -B -b $boot_order_num
  done
  # active no quick bootnum
  case $1 in
    updatebootorder)
      efibootmgr | grep -iE "ip|sata"  | awk -F ' ' '{print $1}' | while read boot_order_line
      do
        boot_order=`echo ${boot_order_line%*\*}`
        boot_order_num=`echo ${boot_order#*Boot}`
        efibootmgr -a -b $boot_order_num
      done
      sed -i 's/STATE=on/STATE=off/g' /etc/cdos-fastboot/cdos-fastboot.conf
      ;;
    *)
      ;;
  esac
}

check_can_use_for_fastboot(){
  rootfstype=`mount | grep "on / " | awk -F ' ' '{print $5}'`
  if [ ! $rootfstype = "ext4" ];then
    return 1
  fi
  if check_disk;then
    return 1
  fi
  if check_swap;then
    return 1
  fi

  if [ -d /dev/mapper ]; then
    if [ `ls /dev/mapper | wc -l` -gt 1 ]; then
      return 1 #disk encryption...
    fi
  fi
  
  if [ -f /etc/linuxoscount ]; then
    if [ `cat /etc/linuxoscount` -gt 0 ]; then
      return 1
    fi
  fi
  
  if check_for_black_list;then
    return 0
  else
    return 1
  fi
}

check_can_use_for_default(){
  rootfstype=`mount | grep "on / " | awk -F ' ' '{print $5}'`
  if [ ! $rootfstype = "ext4" ];then
    exit 1
  fi
  if check_disk;then
    exit 2
  fi
  if check_swap;then
    exit 4
  fi
  
  if [ -d /dev/mapper ]; then
    if [ `ls /dev/mapper | wc -l` -gt 1 ]; then
      return 6 #disk encryption...
    fi
  fi
  
#  LINUXOSCOUNT=`cat /boot/grub/grub.cfg | grep ^"menuentry " | grep " --class " | grep -v " --class windows" | wc -l`
#  if [ $LINUXOSCOUNT -gt 1 ];then
#    exit 5
#  fi
  
  if check_for_white_list;then
    exit 0
  else
    exit 3
  fi
}

check_can_use_for_sys(){
  rootfstype=`mount | grep "on / " | awk -F ' ' '{print $5}'`
  if [ ! $rootfstype = "ext4" ];then
    exit 1
  fi

  if check_disk;then
    exit 2
  fi

  if check_swap;then
    exit 4
  fi

  if [ -d /dev/mapper ]; then
    if [ `ls /dev/mapper | wc -l` -gt 1 ]; then
      return 1 #disk encryption...
    fi
  fi

  if [ -f /etc/linuxoscount ]; then
    if [ `cat /etc/linuxoscount` -gt 0 ]; then
      exit 5
    fi
  fi

  if check_for_black_list;then
    exit 0 # can use, exit by 0
  else
    exit 3
  fi
}

check_can_use_for_user(){
  rootfstype=`mount | grep "on / " | awk -F ' ' '{print $5}'`
  if [ ! $rootfstype = "ext4" ];then
    exit 1
  fi
  if check_disk;then
    exit 2
  fi
  if check_swap;then
    exit 4
  fi
  
  if [ -d /dev/mapper ]; then
    if [ `ls /dev/mapper | wc -l` -gt 1 ]; then
      return 5 #disk encryption...
    fi
  fi  
  
  if check_for_white_list;then
    exit 0
  elif check_for_black_list;then
    exit 0
  else
    exit 3
  fi
}

check_for_black_list(){
  bios_version=`dmidecode -s bios-version`
  vendor=`lspci -n | grep 0300: | head -n1 | awk '{print $3}'`
  bios_list=`cat /usr/share/cdos-fastboot/blacklist/bios`
  # bios_list=`cat ./blacklist/bios`
  bios_check_res=1
  if [[ "$bios_list"x == ""x ]];then
    bios_check_res=1
  else
    while read line
    do
      if [[ "$line"x == "$bios_version"x ]];then
        bios_check_res=0
        break
      fi
    # done < ./blacklist/bios
    done < /usr/share/cdos-fastboot/blacklist/bios
  fi
  vendor_list=`cat /usr/share/cdos-fastboot/blacklist/vendor`
  # vendor_list=`cat ./blacklist/vendor`
  vendor_check_res=1
  if [[ "$vendor_list"x == ""x ]];then
    vendor_check_res=1
  else
    while read line
    do
      if [[ "$line"x == "$vendor"x ]];then
        echo "ada"
        vendor_check_res=0
        break
      fi
    # done < ./blacklist/vendor
    done < /usr/share/cdos-fastboot/blacklist/vendor
  fi
  if [ $bios_check_res -eq 1 ] && [ $vendor_check_res -eq 1 ];then
    return 0
  else
    return 1
  fi
}

check_for_white_list(){
  bios_version=`dmidecode -s bios-version`
  vendor=`lspci -n | grep 0300: | head -n1 | awk '{print $3}'`
  bios_list=`cat /usr/share/cdos-fastboot/whitelist/bios`
  # bios_list=`cat ./whitelist/bios`
  bios_check_res=1
  if [[ "$bios_list"x == "*"x ]];then
    bios_check_res=0
  else
    while read line
    do
      #if [[ "$line"x == "$bios_version"x ]];then
      biosret=`echo "X${bios_version}" | grep "X${line}"`
      if [[ "$biosret"x != x ]];then
        bios_check_res=0
        break
      fi
    # done < ./whitelist/bios
    done < /usr/share/cdos-fastboot/whitelist/bios
  fi
  vendor_list=`cat /usr/share/cdos-fastboot/whitelist/vendor`
  # vendor_list=`cat ./whitelist/vendor`
  vendor_check_res=1
  if [[ "$vendor_list"x == "*"x ]];then
    vendor_check_res=0
  else
    while read line
    do
      if [[ "$line"x == "$vendor"x ]];then
        vendor_check_res=0
        break
      fi
    # done < ./whitelist/vendor
    done < /usr/share/cdos-fastboot/whitelist/vendor
  fi
  if [ $bios_check_res -eq 0 ] || [ $vendor_check_res -eq 0 ];then
    return 0
  else
    return 1
  fi
}

# Check the system hard disk type, HDD or SSD
check_disk(){
  # support HDD & SSD
  return 1  # If only HDD is supported, delete this line

  root=`mount | grep "on / " | awk -F ' ' '{print $1}'`
  tmp=`echo ${root##*/}`
  blo=`echo ${tmp:0:3}`
  disk=0  # Default value

  if [ "$blo"x == "sda"x ]; then
    if [ -f /sys/block/sda/queue/rotational ]; then
      disk=`cat /sys/block/sda/queue/rotational`
    fi
  fi
  # support nvme protocol
  if [ "$blo"x == "nvm"x ]; then
    if [ -f /sys/block/nvme0n1/queue/rotational ]; then
      disk=`cat /sys/block/nvme0n1/queue/rotational`
    fi
  fi

  if [ $disk -eq 0 ];then
    return 0
  else
    return 1
  fi
}

check_swap(){
  #swap_part=`cat /etc/fstab | grep ^"# swap" | sed -n "1,1p" | awk -F ' ' '{print $5}'`
  #For installer change the /etc/fstab content, need change the swap partition detect method.
  swap_part=`blkid  | grep TYPE=\"swap\" | awk -F: '{print $1}'`
  # swap_part="/dev/sda5"
  if [ -z "$swap_part" ];then
    return 0
  else
    return 1
  fi
}

if [ "$UID" -ne 0 ];then
  echo "please run this script in \"root\""
  exit 1
fi

can_use_fastboot_auto_check(){
  rootfstype=`mount | grep "on / " | awk -F ' ' '{print $5}'`
  if [ ! $rootfstype = "ext4" ];then
    return 1
  fi
  if check_disk;then
    return 1
  fi
  if check_swap;then
    return 1
  fi
  
  if [ -d /dev/mapper ]; then
    if [ `ls /dev/mapper | wc -l` -gt 1 ]; then
      return 1 #disk encryption...
    fi
  fi
  
  OSPROBED=""
  if [ -f /usr/bin/os-prober ]; then 
     OSPROBED="`/usr/share/cdos-fastboot/os-probe.sh | tr ' ' '^' | paste -s -d ' '`"
     if [ -z "${OSPROBED}" ] ; then
        echo 0 > /etc/linuxoscount
        echo "Empty os-prober output, nothing to do."
     fi
  fi
  LINUXOSCOUNT=0
  for OS in ${OSPROBED} ; do
    if [ $(echo $OS | grep ".*linux.*"|wc -l) -gt 0 ];then
      LINUXOSCOUNT=$(($LINUXOSCOUNT+1))
    fi
  done

  if [ $LINUXOSCOUNT -gt 0 ];then
    echo $LINUXOSCOUNT > /etc/linuxoscount
    return 1
  else
    echo 0 > /etc/linuxoscount
  fi
  if check_for_black_list;then
    return 0
  else
    return 1
  fi
} 

check_env_for_zxd(){
  root=`mount | grep "on / " | awk -F ' ' '{print $1}'`
  tmp=`echo ${root##*/}`
  blo=`echo ${tmp:0:3}`
  disk=1  # Default value

  # support nvme protocol
  if [ "$blo"x == "nvm"x ]; then
    if [ -f /sys/block/nvme0n1/queue/rotational ]; then
      disk=`cat /sys/block/nvme0n1/queue/rotational`
    fi
  fi

  if [ $disk -eq 0 ];then
    echo "0"
    return 0
  fi

  echo "1"
  return 1
}


# 用于UEFI模式安装的系统下新efiloader配置文件的修改，用于加入或者删除配置文件的resume参数
# with-resume/without-resume应当仅在系统关机以及系统初开启的过程中被使用
set_fastboot_efiloader_config(){
  cfg_file="/boot/efi/EFI/Boot/cdosefi.cfg"
  if [ -f "$cfg_file" ];then
    swap_part=`blkid  | grep TYPE=\"swap\" | awk -F: '{print $1}'`
    check_result=$(echo `cat $cfg_file | grep "resume="`)
    case $1 in
      update)
	set_boot_dir
        kernel=`get_kernel_name`
        if [ ! -z "$kernel" ];then
          cp $kernel /boot/efi/EFI/cdos/vmlinuz.efi
        else
          set_fastboot_off
          return 1
	fi
        root_part=`mount | grep "on / " | awk -F ' ' '{print $1}'`
        swap_part=`blkid  | grep TYPE=\"swap\" | awk -F: '{print $1}'`
        rootfstype=`mount | grep "on / " | awk -F ' ' '{print $5}'`
        grub_cmd_linux=`get_grub_cmd_linux | sed -e 's/hibernate=nocompress//g' | sed -e 's/hibernation=nocompress//g' | sed -e 's/init=\/sbin\/e4rat-preload//g'`

        echo "-f 0:\EFI\cdos\vmlinuz.efi root=$root_part ro rootfstype=$rootfstype $grub_cmd_linux resume=$swap_part raid=noautodetect loglevel=0 systemd.show_status=0" > $cfg_file
        ;;
      with-resume)
        if [ "$check_result" == "" ]; then          
          echo $(cat $cfg_file | sed 's/\n//g') "resume="$swap_part > $cfg_file
        fi
        ;;
      without-resume)
        if [ "$check_result" != "" ]; then
	  str="resume="$swap_part
	  echo $(cat $cfg_file | sed "s#$str##g") > $cfg_file
        fi
        ;;
    esac
  fi
}

# JUST FOR EFILOADER OF FASTBOOT !
efiloader_set(){
  if [ -d /boot/efi/EFI/Boot ]
  then
    case $1 in
      add)
        if [ -f /usr/share/cdos-fastboot/cdosefi.cfg ] && [ -f /usr/share/cdos-fastboot/efilinux.efi ];then
          cp /usr/share/cdos-fastboot/cdosefi.cfg /boot/efi/EFI/Boot/
          mv /boot/efi/EFI/Boot/bootx64.efi /boot/efi/EFI/Boot/bootx64.efi.bak
          cp /usr/share/cdos-fastboot/efilinux.efi /boot/efi/EFI/Boot/bootx64.efi
          set_fastboot_efiloader_config update
        fi
	;;
      remove)
	if [ -f /boot/efi/EFI/Boot/bootx64.efi.bak ];then
          mv /boot/efi/EFI/Boot/bootx64.efi.bak /boot/efi/EFI/Boot/bootx64.efi
        fi
        if [ -f /boot/efi/EFI/Boot/cdosefi.cfg ];then
          rm -f /boot/efi/EFI/Boot/cdosefi.cfg
        fi
	;;
    esac
  fi
}

# TODO: 未保证可维护性，对外接口中不应包含主要方案策略相关的实现!!!
case $1 in
  on)
    set_fastboot_on
    # if [ -d /sys/firmware/efi ];then
    #   if check_env_for_zxd
    #   then
    #     efiloader_set add
    #   else
    #     set_efi_quick_start_on noinitrd updatebootorder
    #   fi
    #   cp -f /usr/share/cdos-fastboot/cdos-loader /boot/efi/EFI/cdos/cdos-loader.efi
    # fi
    if [ -f /etc/modprobe.d/blacklist_disc_gfx.conf ] && [ -d /usr/share/cdos-fastboot/ ]; then
      mv -f /etc/modprobe.d/blacklist_disc_gfx.conf /usr/share/cdos-fastboot/
    fi
    if [ -f /etc/init.d/cdos-antivirus-daemon ] && [ -d /usr/share/cdos-fastboot/ ]; then
      mv -f /etc/init.d/cdos-antivirus-daemon /usr/share/cdos-fastboot/
    fi
    #systemctl enable resetefi
    ;;
  off)
    # if [ -d /sys/firmware/efi ];then
    #   if check_env_for_zxd
    #   then
    #     efiloader_set remove
    #   else
    #     set_efi_quick_start_off updatebootorder
    #   fi
    # fi
    set_fastboot_off
    count_file="/var/log/counts.conf"
    if [ -f $count_file ];then
      rm $count_file
    fi
    if [ -f /usr/share/cdos-fastboot/blacklist_disc_gfx.conf ] && [ -d /etc/modprobe.d/ ]; then
      mv -f /usr/share/cdos-fastboot/blacklist_disc_gfx.conf /etc/modprobe.d/
    fi
    if [ -f /usr/share/cdos-fastboot/cdos-antivirus-daemon ] && [ -d /etc/init.d/ ]; then
      mv -f /usr/share/cdos-fastboot/cdos-antivirus-daemon /etc/init.d/
    fi
    #systemctl disable resetefi
    ;;
  resetbootorder)
    # reset_efi_bootorder
    ;;
  resetstate)
    reset_fastboot_state
    ;;
  check)
    check_can_use_for_user
    ;;
  checkfordefault)
    check_can_use_for_default
    ;;
  checkforsys)
    check_can_use_for_sys
    ;;
  autocheck)
    # now, JUST FOR fastboot_env_check.service
    sleep 6
    if [ -d /sys/firmware/efi ]; then
      set_fastboot_efiloader_config with-resume
      # if check_env_for_zxd
      # then
      #   set_fastboot_efiloader_config with-resume
      # fi
    fi

    sleep 3
    if [[ `cat /etc/cdos-fastboot/cdos-fastboot.conf` =~ "STATE=on"  ]] && ! can_use_fastboot_auto_check;then
      set_fastboot_off
      # if check_env_for_zxd;then
      #   efiloader_set remove
      # fi
    fi
    ;;
  checkenv) # for lightdm.
    check_env_for_zxd
    ;;
  set-fastboot-efiloader) # for lightdm.
    # $2: resume | without-resume
    set_fastboot_efiloader_config $2
    ;;
  *)
    echo $"Usage: sudo bash $0 {on|off|resetbootorder|resetstate|check|checkfordefault|checkforsys}"
    ;;
esac
