#!/bin/python
#-*- coding: utf-8 -*-

# Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
# 
# This software is published by the license of CPU-OS License, you can use and 
# distibuted this software under this License. See CPU-OS Liense for more detail.
#
# You should have receive a copy of CPU-OS License. If not, please contact us 
# by email <support_os@cpu-os.ac.cn> 

import os
import shutil
import subprocess
import re
import commands
from optparse import OptionParser

parser = OptionParser()
parser.add_option("-f", "--file", action = "store" , type = "string", dest = "file", help = "config file in /etc/default", metavar = "config")
parser.add_option("-r", "--root", action = "store" , type = "string", dest = "root", help = "config file in /etc/default", metavar = "root partition")
parser.add_option("-s", "--state", action = "store" , type = "string", dest = "state", help = "do something for fastboot state", metavar = "STATE")
#解析参数
(options, args) = parser.parse_args()

class Result:
    __error = None
    __value = None

    def __init__(self, error, value):
        self.__error = error
        self.__value = value

    def getError(self):
        return self.__error

    def getValue(self):
        return self.__value

def get_grub_env_from_file():
    grub_env_json = {}
    grub_config_path = '/etc/default/grub'
    grub_config_dir = '/etc/default/grub.d/'

    # get config form '/etc/default/grub'
    if not os.path.isfile( grub_config_path):
        return Result( 'Error: /etc/default/grub is not exist', None)
    get_config_from_file_res = get_config_from_file( grub_config_path)
    if get_config_from_file_res.getValue():
        grub_env_json = get_config_from_file_res.getValue()
    # print grub_env_json
    # get config form '/etc/default/grub.d'
    if os.path.isdir( grub_config_dir):
        for file_name in sorted( os.listdir( grub_config_dir)):
            if not file_name.endswith(".cfg"):
                continue
            get_config_from_file_res = get_config_from_file( grub_config_dir + file_name)
            if get_config_from_file_res.getValue():
                tmp_env_json = get_config_from_file_res.getValue()
                # print tmp_env_json
                for env_name in tmp_env_json:
                    if tmp_env_json[env_name].startswith( "\"$" + env_name):
                        # print tmp_env_json[env_name]
                        # print tmp_env_json[env_name][len(env_name)+ 2: -1]
                        grub_env_json[ env_name] = grub_env_json[ env_name][:-1] + tmp_env_json[env_name][len(env_name)+ 2:]
                    else:
                        grub_env_json[ env_name] = tmp_env_json[env_name]
                # print grub_env_json
    # print grub_env_json
    return grub_env_json

def get_config_from_file(config_file_name):
    config_file_json = {}
    try:
        grub_config_msgs = open(config_file_name,'r').read().split('\n')
        for grub_env in grub_config_msgs:
            if ( grub_env != '') and ( not grub_env.startswith('#')):
                grub_env_name = grub_env.split('=')[0]
                grub_env_msg = grub_env[ len(grub_env_name) + 1:]
                config_file_json[ grub_env_name] = grub_env_msg
        return Result(None, config_file_json)
    except Exception as e:
        return Result('Exception: ' + str(e), None)

def get_10_linux_config(file_path="/etc/grub.d/10_linux"):
    ubuntu_recovery_value = vt_handoff_value = init_paths = None
    config_json = {}
    for line in open(file_path).read().split("\n"):
        if line.startswith("ubuntu_recovery="):
            ubuntu_recovery_value = line.split("=")[-1]
        if line.startswith("vt_handoff="):
            vt_handoff_value = line.split("=")[-1]
        if line.startswith("SUPPORTED_INITS="):
            init_paths = line.split("=")[-1][1:-1]
        if (ubuntu_recovery_value is not None) and (vt_handoff_value is not None) and (init_paths is not None):
            break
    if (ubuntu_recovery_value is None) or (ubuntu_recovery_value == "0"):
        config_json["recovery"] = False
    else:
        config_json["recovery"] = True
    if (vt_handoff_value is None) or (vt_handoff_value == "0"):
        config_json["vt_handoff"] = False
    else:
        config_json["vt_handoff"] = True
    config_json["inits"] = {}
    if init_paths is not None:
        for init_path in init_paths.split():
            config_json["inits"][init_path.split(":")[0]] = init_path.split(":")[1]
    return config_json

def get_root_part():
    if options.root is not None:
        return options.root
    else:
        root_part = root_part_name = root_part_id = root_part_type = None
        mount_infos = subprocess.check_output("mount").split("\n")
        for info in mount_infos:
            info_list = info.split()
            if len( info_list) < 3:
                continue
            if "/" == info_list[2]:
                root_part = info_list[0]
                root_part_name = re.split(r"[0-9]", root_part)[0]
                root_part_id = root_part[len(root_part_name):]
                root_part_type = info_list[4]
        return root_part

def update_boot_grub_cfg(grub_env, menu_id=None, cmd_linux_defalut_line="\"\"", timeout=None, boot_grub_cfg_file="/boot/grub/grub.cfg", update_menu=False, update_cmdline=False, update_timeout=False, fastboot_state=False):
    try:
        root_part = get_root_part()
        # get config in 10_linux
        up_cfg = get_10_linux_config()
        # get default cmdline
        if "GRUB_CMDLINE_LINUX_DEFAULT" in grub_env:
            cmd_linux_defalut_line = grub_env["GRUB_CMDLINE_LINUX_DEFAULT"]
        else:
            cmd_linux_defalut_line = ""
        if cmd_linux_defalut_line.endswith("\n"):
            cmd_linux_defalut_line = cmd_linux_defalut_line[:-1]
        if cmd_linux_defalut_line.startswith("\"") or cmd_linux_defalut_line.startswith("\'"):
            cmd_linux_defalut_line = cmd_linux_defalut_line[1:]
        if cmd_linux_defalut_line.endswith("\"") or cmd_linux_defalut_line.endswith("\'"):
            cmd_linux_defalut_line = cmd_linux_defalut_line[:-1]
        # get cmdline
        if "GRUB_CMDLINE_LINUX" in grub_env:
            cmd_linux_line = grub_env["GRUB_CMDLINE_LINUX"]
        else:
            cmd_linux_line = ""
        if cmd_linux_line.startswith("\"") or cmd_linux_line.startswith("\'"):
            cmd_linux_line = cmd_linux_line[1:]
        if cmd_linux_line.endswith("\"") or cmd_linux_line.endswith("\'"):
            cmd_linux_line = cmd_linux_line[:-1]
        # get recovery title
        if "GRUB_RECOVERY_TITLE" in grub_env:
            recovery_title = grub_env["GRUB_RECOVERY_TITLE"]
        else:
            recovery_title = "recovery mode"
        # get default init type
        try:
            local_init_type = os.readlink("/sbin/init")
        except Exception as e:
            local_init_type = None

        # get init types not default
        init_types = []
        for init_type in up_cfg["inits"]:
            if (local_init_type is None) or (not up_cfg["inits"][init_type] == local_init_type):
                init_types.append(init_type)
        recovery_args = ""
        if os.path.isfile("/lib/recovery-mode/recovery-menu"):
            recovery_args += "recovery"
        else:
            recovery_args += "single"
        '''
        if up_cfg["recovery"]:
            recovery_args += " nomodeset"
        '''
        write_data = ""

        this_init_type = ""
        update_menu_flag = update_cmdline_flag = update_timeout_flag = False
        recovery_flag = init_flag = default_flag = False
        for line in open(boot_grub_cfg_file).read().split("\n"):
            if update_menu and ("BEGIN /etc/grub.d/00_header" in line):
                update_menu_flag = True
            elif update_menu and update_menu_flag and ("set default=" in line) and (not "{next_entry}" in line):
                write_data += line.split("=")[0] + "=" + menu_id
                update_menu_flag = False
                continue
            elif update_timeout and ("timeout_style=menu" in line):
                update_timeout_flag = True
            elif update_timeout and update_timeout_flag and ("set timeout=" in line):
                write_data += line.split("=")[0] + "=" + timeout
                continue
            elif update_timeout and ("END /etc/grub.d/00_header" in line):
                # print "end update_timeout_flag"
                update_timeout = update_timeout_flag = False
            elif update_cmdline and ("BEGIN /etc/grub.d/10_linux" in line):
                update_cmdline_flag = True
            elif update_cmdline and ("END /etc/grub.d/10_linux" in line):
                update_cmdline_flag = False
            elif update_cmdline and update_cmdline_flag:
                if line.startswith("menuentry") or line.startswith("\tmenuentry"):
                    menu_title = line.split("\'")[1]
                    if recovery_title in menu_title:
                        recovery_flag = True
                    for tmp_init_type in init_types:
                        if tmp_init_type in menu_title:
                            init_flag = True
                            this_init_type = tmp_init_type
                            break
                #elif line.startswith("\tlinux") or line.startswith("\t\tlinux") or line.startswith("       linux"):
                elif line.startswith("\tlinux") or line.startswith("\t\tlinux") or line.find(" splash ")!=-1:
                    #  快速启动开启，更换为root=/dev/...格式
                    if fastboot_state:
                        root_path_str = "root="+root_part
                        line = re.sub(r'root=UUID=[^\s]*',root_path_str, line)
                        '''
                        tmp_basic_line = line.split("ro ")[0]+ "ro "
                        basic_line = line.split("linux")[0]
                        for tmp_line in tmp_basic_line.split():
                            if tmp_line == "linux":
                                basic_line += "linux\t"
                            elif not tmp_line.startswith("root="):
                                basic_line += tmp_line+" "
                            else:
                                basic_line += "root="+root_part+" "
                        
                    else:
                        basic_line = line.split("ro ")[0]+ "ro "
                    '''
                    if recovery_flag:
                        #basic_line += recovery_args + " " + cmd_linux_line
                        if up_cfg["recovery"] and " nomodeset" not in line:
                            line += " nomodeset"
                        recovery_flag = False
                    elif init_flag:
                        '''
                        basic_line += cmd_linux_line + " " + cmd_linux_defalut_line
                        if up_cfg["vt_handoff"] and ("splash" in cmd_linux_defalut_line):
                            basic_line += " $vt_handoff"
                        basic_line += " init=" + up_cfg["inits"][this_init_type]
                        '''
                        init_flag = False
                    else:
                        # basic_line += cmd_linux_line + " " + cmd_linux_defalut_line
                        if fastboot_state:
                            (status_sw, swap_part) = commands.getstatusoutput ("cat /etc/fstab | grep ^'# swap' | sed -n '1,1p' | awk -F ' ' '{print $5}'")
                            (status_rft, rootfstype) = commands.getstatusoutput ("mount | grep 'on / ' | awk -F ' ' '{print $5}'")
                            if not status_sw and not status_rft:
                                if swap_part != "":
                                    '''
                                    basic_line = basic_line.replace ("hibernate=nocompress"," ")
                                    basic_line += " resume=" + swap_part + " rootfstype=" + rootfstype + " raid=noautodetect "
                                    '''
                                    line = line.replace (" hibernate=nocompress"," ")
                                    # line += " resume=" + swap_part + " rootfstype=" + rootfstype + " raid=noautodetect "
                                    if " resume=" not in line:
                                        line += " resume=" + swap_part
                                    else :
                                        line = re.sub(r'resume=[^\s]*', "resume=" + swap_part, line)
                                    if " rootfstype=" not in line:
                                        line += " rootfstype=" + rootfstype
                                    else :
                                        line = re.sub(r'rootfstype=[^\s]*', "rootfstype=" + rootfstype, line)
                                    if " raid=" not in line:
                                        line += " raid=noautodetect"
                                    else :
                                        line = re.sub(r'raid=[^\s]*', "raid=noautodetect", line)
                        else :
                            if "hibernate=nocompress" in cmd_linux_defalut_line and " hibernate=" not in line:
                                line += " hibernate=nocompress"
                            line = re.sub(r' resume=[^\s]*', '', line)
                            line = re.sub(r' rootfstype=[^\s]*', '', line)
                            line = re.sub(r' raid=[^\s]*', '', line)
                        if up_cfg["vt_handoff"] and ("splash" in cmd_linux_defalut_line):
                            # basic_line += " $vt_handoff"
                            if "$vt_handoff" not in line:
                                line += " $vt_handoff"
                    # write_data += basic_line + "\n"
                    write_data += line + "\n"
                    continue
                elif (fastboot_state == True) and (line.startswith("\tinitrd") or line.startswith("\t\tinitrd")):
                    write_data += line.split("initrd")[0] + "# " + line[len(line.split("initrd")[0]):]+"\n"
                    continue
                elif (fastboot_state == False) and (line.startswith("\t# initrd") or line.startswith("\t\t# initrd")):
                    write_data += line.split("# initrd")[0] + line[len(line.split("# initrd")[0]) + 2:]+"\n"
                    continue
            write_data += line + "\n"

        open(boot_grub_cfg_file+"_bk","w").write(write_data)
        shutil.copyfile(boot_grub_cfg_file+"_bk", boot_grub_cfg_file)
        return True
    except Exception as e:
        return False

def set_fastboot_for_grub_cfg_in_boot(state):
    boot_cfg_file = "/boot/grub/grub.cfg"
    grub_env = get_grub_env_from_file()
    update_boot_grub_cfg(grub_env, boot_grub_cfg_file=boot_cfg_file, update_cmdline=True, fastboot_state=state)

if __name__ == "__main__":
    if options.state is not None:
        if options.state == "on":
            set_fastboot_for_grub_cfg_in_boot(True)
        elif options.state == "off":
            set_fastboot_for_grub_cfg_in_boot(False)
